/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry.contrib.table.model.sql;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * @version $Id: ResultSetIterator.java,v 1.1 2003/03/05 23:03:20 hlship Exp $
 * @author mindbridge
 */
public class ResultSetIterator implements Iterator
{
	private static final Log LOG = LogFactory.getLog(ResultSetIterator.class);

	private ResultSet m_objResultSet;
	private boolean m_bFetched;
	private boolean m_bAvailable;

	public ResultSetIterator(ResultSet objResultSet)
	{
		m_objResultSet = objResultSet;
		m_bFetched = false;
	}

	/**
	 * @see java.util.Iterator#hasNext()
	 */
	public synchronized boolean hasNext()
	{
        if (getResultSet() == null) return false;
        
		if (!m_bFetched)
		{
			m_bFetched = true;

			try
			{
				m_bAvailable = !getResultSet().isLast();
			}
			catch (SQLException e)
			{
				LOG.warn(
					"SQLException while testing for end of the ResultSet",
					e);
				m_bAvailable = false;
			}

			if (!m_bAvailable)
				notifyEnd();
		}

		return m_bAvailable;
	}

	/**
	 * @see java.util.Iterator#next()
	 */
	public synchronized Object next()
	{
		ResultSet objResultSet = getResultSet();

		try
		{
			if (!objResultSet.next())
				return null;
		}
		catch (SQLException e)
		{
			LOG.warn("SQLException while iterating over the ResultSet", e);
			return null;
		}

		m_bFetched = false;
		return objResultSet;
	}

	/**
	 * @see java.util.Iterator#remove()
	 */
	public void remove()
	{
		try
		{
			getResultSet().deleteRow();
		}
		catch (SQLException e)
		{
			LOG.error("Cannot delete record", e);
		}
	}

	/**
	 * Returns the resultSet.
	 * @return ResultSet
	 */
	public ResultSet getResultSet()
	{
		return m_objResultSet;
	}

	protected void notifyEnd()
	{
	}

}
