/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry.util;

import java.util.HashMap;
import java.util.Map;

/**
 *  Used to "uniquify" names within a given context.  A base name
 *  is passed in, and the return value is the base name, or the base name
 *  extended with a suffix to make it unique.
 *
 *  @author Howard Lewis Ship
 *  @version $Id: IdAllocator.java,v 1.3 2003/05/13 16:49:33 hlship Exp $
 *  @since 3.0
 *
 **/

public class IdAllocator
{
    private Map _generatorMap = new HashMap();

    private static class NameGenerator
    {
        private String _baseId;
        private int _index;

        NameGenerator(String baseId)
        {
            _baseId = baseId + "$";
        }

        public String nextId()
        {
            return _baseId + _index++;
        }
    }

    /**
     *  Allocates the id.  Repeated calls for the same name will return
     *  "name", "name_0", "name_1", etc.
     * 
     **/

    public String allocateId(String name)
    {
        NameGenerator g = (NameGenerator) _generatorMap.get(name);
        String result = null;

        if (g == null)
        {
            g = new NameGenerator(name);
            result = name;
        }
        else
            result = g.nextId();

        // Handle the degenerate case, where a base name of the form "foo$0" has been
        // requested.  Skip over any duplicates thus formed.
        
        while (_generatorMap.containsKey(result))
            result = g.nextId();

        _generatorMap.put(result, g);

        return result;
    }
    
    /**
     *  Clears the allocator, resetting it to freshly allocated state.
     * 
     **/
    
    public void clear()
    {
        _generatorMap.clear();
    }
}
